# Auswahlmenü für Dateiselektion vor Rechtschreibprüfung
#
# Copyright (C) 2004  G. Lamprecht, W. Lotz, R. Weibezahn; IWD, Bremen University

proc rs  {} {



global vv redateiliste dir

set redateiliste ""
foreach i  [DelWinNTFiles [lsort [glob -nocomplain -- *]]] {
  if { ([file isfile $i]) && ([string match "*.tex" $i]) } {
    set redateiliste "${redateiliste} $i"
  }
}

# ggfs. (falls sinnvoll und Bedarf!) später Selektionsmenü
#
#  
#      "OK"  "Hilfen"  "Löschen Textfeld"
#
#      Dateiliste:
#      -----------------------      --------------------------------
#      |a1.tx              | |      |ausgewählt sind folgende   |  |
#      |a3.tex             | |      |Dateien:                   |  |
#      |a5.tex             | |      |  a2.tex                   |  |
#      |a6.tex             | |      |  a4.tex                   | ^|
#      |                   | |      |                           | v|
#      |                   |^|      |                           |  |
#      |                   |v|      |                           |  |
#      |                   | |      |                           |  |
#      |                   | |      |                           |  |
#      |                   | |      |                           |  |
#      |                   | |      |                           |  |
#      |                   | |      |                           |  |
#      |                   | |      |                           |  |
#      |                   | |      |                           |  |
#      |                   | |      |                           |  |
#      |                   | |      |                           |  |
#      |                   | |      |                           |  |
#      |                   | |      |                           |  |
#      |                   | |      |                           |  |
#      |                   | |      |                           |  |
#      -----------------------      --------------------------------
#      <------- 30 ---------->      <----------- 40 --------------->
#
#      Ausgewählte Dateien          Bei jedem Neuzugang (durch Auswahl
#      verschwinden aus dieser      per Anklicken in linker Liste)
#      Liste sofort und werden      updating!
#      sofort nach rechts 
#      übernommen.

}
