# display local news, edited in file $localnews   
#
# Copyright (C) 2004  G. Lamprecht, W. Lotz, R. Weibezahn; IWD, Bremen University

proc newsfile {f austext} {

  upvar 1 $austext a

  global  vv localnews

  writescr0 $f ""
  set a ""


  if {![file exists $localnews]} {set a "${a}$vv(ne2)\n"; return};# $localnews doesn't exist!

  if {![file isfile $localnews]} {# $localnews is no regular file!
    set a "${a}***** $vv(aus5) \"$localnews\"  $vv(aus6)\n"; mybell 1; return
  }


  writescr $f a "${a}[datime] $vv(ne1)  $localnews\n\n"

  catFile $localnews $f absolutely      

}
